/*
 * SourceMod Radio
 *
 * Adds a radio player to SourceMod that supports direct station links or streaming audio
 *
 * Coded by dubbeh - www.dubbeh.net
 *
 * Licensed under the GPLv3
 *
 */


void PrintPrimaryGenreList(int iClient)
{
	int iIndex = 0, iCount = 0;
	char szGenreName[64] = "";
	char szBuffer[512] = "";
	Genre genre;
	
	for (iIndex = 0, iCount = 0; iIndex < g_Genres.Count; iIndex++) {
		genre = g_Genres.Items(iIndex);
		
		if (genre.IsValid && genre.m_iParentID == 0) {
			genre.GetName(szGenreName, sizeof(szGenreName));
			
			if ((iIndex + 2) == g_Genres.Count) {
				StrCat(szBuffer, sizeof(szBuffer), szGenreName);
			} else {
				StrCat(szBuffer, sizeof(szBuffer), szGenreName);
				StrCat(szBuffer, sizeof(szBuffer), ", ");
			}
			
			iCount++;
			
			if (iCount == 10) {
				CReplyToCommand(iClient, szBuffer);
				clear_string(szBuffer, strlen(szBuffer));
				iCount = 0;
			}
		}
	}
	
	if (iCount < 10) {
		CReplyToCommand(iClient, szBuffer);
	}
}

void clear_string(char[] szBuffer, int iLen)
{
	int iIndex = 0;
	
	for (iIndex = 0; iIndex <= iLen; iIndex++) {
		szBuffer[iIndex] = 0;
	}
}

int FindPrimaryGenreIndexByName(char[] szGenreName)
{
	char szCompareGenreName[64] = "";
	int iIndex = 0;
	Genre genre;
	
	for (iIndex = 0; iIndex < g_Genres.Count; iIndex++) {
		genre = g_Genres.Items(iIndex);
		if (genre.IsValid && genre.m_iParentID == 0) {
			genre.GetName(szCompareGenreName, sizeof(szCompareGenreName));
			if (!strcmp(szGenreName, szCompareGenreName, false))
				return iIndex;
		}
	}
	
	return 0;
}

void BrowseURL(int iClient, char[] szURL)
{
	if (g_EngineVersion == Engine_CSGO)
		CReplyToCommand(iClient, "Browse command unsupported in CS:GO");
	else
		ShowMOTDPanel(iClient, "SourceMod Browse", szURL, MOTDPANEL_TYPE_URL);
}



void LoadStation(int iClient, char[] szStationID, bool bCustomShoutCast)
{
	int iStationID = 0;
	char szBuffer[MAX_MOTD_URL_SIZE] = "";
	char szStreamURL[128] = "";
	char szStationName[64] = "";
	Station station;
	CustomStation custstation;
	
	if (((iStationID = StringToInt(szStationID)) > 0) && IsClientConnected(iClient) && IsClientInGame(iClient))
	{
		if (g_iRadioPlayType == RADIO_PLAY_TYPE_SHOUTCAST)
		{
			if (bCustomShoutCast)
			{
				custstation = g_CustStations.Items(GetCustomStationIndexFromID(iStationID));
				if (custstation.IsValid)
				{
					custstation.GetStreamURL(szStreamURL, sizeof(szStreamURL));
					if (szStreamURL[0])
					{
						ShowMOTDFromURL(iClient, RadioVolumeWrapper(iClient, szStreamURL, GetClientStationVolume(iClient)));
						SetClientCookiesForStation(iClient, szStationID, bCustomShoutCast);
						
						if (g_cVarStationAdvert.IntValue)
						{
							custstation.GetName(szStationName, sizeof(szStationName));
							AdvertiseStationChoice(iClient, szStationName);
						}
					} else {
						LogMessage(g_szNoStreamURLSet, custstation.m_iID);
					}
				} else {
					LogMessage(g_szInvalidStationIndex);
				}
			} else {
				station = g_Stations.Items(GetStationIndexFromID(iStationID));
				if (station.IsValid)
				{
					Format(szBuffer, sizeof(szBuffer), "%s?sid=%d&mod=%d&vol=%s", g_szShoutCastURL, station.m_iID, g_EngineVersion, GetClientStationVolume(iClient));
					ShowMOTDFromURL(iClient, szBuffer);
					SetClientCookiesForStation(iClient, szStationID, bCustomShoutCast);
					if (g_cVarStationAdvert.IntValue)
					{
						station.GetName(szStationName, sizeof(szStationName));
						AdvertiseStationChoice(iClient, szStationName);
					}
				} else {
					LogMessage(g_szInvalidStationIndex);
				}
			}
		}
		else if (g_iRadioPlayType == RADIO_PLAY_TYPE_DIRECT)
		{
			station = g_Stations.Items(GetStationIndexFromID(iStationID));
			if (station.IsValid)
			{
				station.GetStreamURL(szStreamURL, sizeof(szStreamURL));
				if (szStreamURL[0])
				{
					ShowMOTDFromURL(iClient, szStreamURL);
					SetClientCookiesForStation(iClient, szStationID, bCustomShoutCast);
					if (g_cVarStationAdvert.IntValue)
					{
						station.GetName(szStationName, sizeof(szStationName));
						AdvertiseStationChoice(iClient, szStationName);
					}
				} else {
					LogMessage(g_szNoStreamURLSet, station.m_iID);
				}
			} else {
				LogMessage(g_szInvalidStationIndex);
			}
		}
		else if (g_iRadioPlayType == RADIO_PLAY_TYPE_VOLUME_CONTROL)
		{
			station = g_Stations.Items(GetStationIndexFromID(iStationID));
			if (station.IsValid)
			{
				station.GetStreamURL(szStreamURL, sizeof(szStreamURL));
				if (szStreamURL[0])
				{
					ShowMOTDFromURL(iClient, RadioVolumeWrapper(iClient, szStreamURL, GetClientStationVolume(iClient)));
					SetClientCookiesForStation(iClient, szStationID, bCustomShoutCast);
					if (g_cVarStationAdvert.IntValue)
					{
						station.GetName(szStationName, sizeof(szStationName));
						AdvertiseStationChoice(iClient, szStationName);
					}
				} else {
					LogMessage(g_szNoStreamURLSet, station.m_iID);
				}
			} else {
				LogMessage(g_szInvalidStationIndex);
			}
		}
	}
}

void SetClientCookiesForStation(int iClient, char[] szStationID, bool bIsCustom)
{
	SetClientCookie(iClient, g_hCookieSavedStation, szStationID);
	if (bIsCustom)
		SetClientCookie(iClient, g_hCookieCustomShoutCast, "1");
	else
		SetClientCookie(iClient, g_hCookieCustomShoutCast, "0");
}

void AdvertiseStationChoice(int iClient, char[] szStationName)
{
	char szClientName[MAX_NAME_LENGTH + 1];
	
	GetClientName(iClient, szClientName, sizeof(szClientName));
	CPrintToChatAll("%T", "Started Listening", LANG_SERVER, szClientName, szStationName);
}

char GetClientStationVolume(int iClient)
{
	char szStationVolume[12] = "";
	
	GetClientCookie(iClient, g_hCookieSavedVolume, szStationVolume, sizeof(szStationVolume));
	if (!szStationVolume[0])
	{
		FloatToString(g_cVarInitialConnectVolume.FloatValue, szStationVolume, sizeof(szStationVolume));
		SetClientCookie(iClient, g_hCookieSavedVolume, szStationVolume);
	}
	
	return szStationVolume;
}

void ShowMOTDFromURL(int iClient, char[] szURL)
{
	if (!IsClientConnected(iClient) || !IsClientInGame(iClient))
		return;
	
	if (g_EngineVersion == Engine_CSGO)
		LoadMOTDPanel(iClient, PLUGIN_NAME, szURL, true);
	else
		ShowMOTDPanel(iClient, PLUGIN_NAME, szURL, MOTDPANEL_TYPE_URL);
}

char RadioVolumeWrapper(int iClient, char[] szStationURL, char[] szVolume)
{
	char szURL[256] = "";
	
	Format(szURL, sizeof(szURL), "%s?url=%s&volume=%s", g_szVolumeWrapperURL, szStationURL, szVolume);
	IsURLTooLong(iClient, szURL);
	return szURL;
}

void IsURLTooLong(int iClient, char[] szURL)
{
	if (IsClientConnected(iClient) && IsClientInGame(iClient))
	{
		if (strlen(szURL) > MAX_MOTD_URL_SIZE)
		{
			LogMessage("[SM-RADIO] Warning: URL \"%s\" exceeds the 192 bytes limit and will end up truncated.", szURL);
			CPrintToChat(iClient, "%T", "Radio URL Too Long", LANG_SERVER);
		}
	}
}


void LoadOffPage(int iClient)
{
	char szClientName[MAX_NAME_LENGTH + 1] = "";
	
	if (IsClientConnected(iClient) && IsClientInGame(iClient))
	{
		LoadMOTDPanel(iClient, "SourceMod Radio", g_szRadioOffPage, true);
		SetClientCookie(iClient, g_hCookieSavedStation, "");
		
		if (g_cVarStationAdvert.IntValue)
		{
			GetClientName(iClient, szClientName, sizeof(szClientName));
			CPrintToChatAll("%T", "Stopped Listening", LANG_SERVER, szClientName);
		}
	}
}

void LoadMOTDPanel(int iClient, char[] szTitle, char[] szPage, bool bHidden)
{
	if (!IsClientConnected(iClient) || !IsClientInGame(iClient))
		return;
	
	Handle kv = INVALID_HANDLE;
	
	kv = CreateKeyValues("data");
	
	KvSetString(kv, "title", szTitle);
	KvSetNum(kv, "type", MOTDPANEL_TYPE_URL);
	KvSetString(kv, "msg", szPage);
	KvSetNum(kv, "cmd", 0); //http://forums.alliedmods.net/showthread.php?p=1220212
	ShowVGUIPanel(iClient, "info", kv, bHidden);
	CloseHandle(kv);
}

int GetGenreIndexFromID(int iGenreID)
{
	int iIndex = 0;
	Genre genre;
	
	for (iIndex = 0; iIndex < g_Genres.Count; iIndex++) {
		genre = g_Genres.Items(iIndex);
		if (!genre.IsValid)
			continue;
		else if (genre.m_iID == iGenreID)
			return iIndex;
	}
	
	return 0;
}

int GetCustomGenreIndexFromID(int iGenreID)
{
	int iIndex = 0;
	CustomGenre custgenre;
	
	for (iIndex = 0; iIndex < g_CustGenres.Count; iIndex++) {
		custgenre = g_CustGenres.Items(iIndex);
		if (!custgenre.IsValid)
			continue;
		else if (custgenre.m_iID == iGenreID)
			return iIndex;
	}
	
	return 0;
}

int GetStationIndexFromID(int iStationID)
{
	int iIndex = 0;
	Station station;
	
	for (iIndex = 0; iIndex < g_Stations.Count; iIndex++) {
		station = g_Stations.Items(iIndex);
		
		if (!station.IsValid)
			continue;
		else if (iStationID == station.m_iID)
			return iIndex;
	}
	
	return 0;
}

int GetCustomStationIndexFromID(int iStationID)
{
	int iIndex = 0;
	CustomStation custstation;
	
	for (iIndex = 0; iIndex < g_CustStations.Count; iIndex++) {
		custstation = g_CustStations.Items(iIndex);
		
		if (!custstation.IsValid)
			continue;
		else if (iStationID == custstation.m_iID)
			return iIndex;
	}
	
	return 0;
}
