/*
 * SourceMod Radio
 *
 * Adds a radio player to SourceMod that supports direct station links or streaming audio
 *
 * Coded by dubbeh - www.dubbeh.net
 *
 * Licensed under the GPLv3
 *
 */


void CreateRadioMenus()
{
	CreateRadioBaseMenu();
	
	if (g_iRadioPlayType == RADIO_PLAY_TYPE_VOLUME_CONTROL || g_iRadioPlayType == RADIO_PLAY_TYPE_SHOUTCAST)
		CreateRadioVolumeMenu();
	
	if (g_bPlayListFound)
	    CreateGenreAndStationMenus();
	
	// ShoutCast custom stations menu initialization
	if (g_bShoutCastCustom && g_iRadioPlayType == RADIO_PLAY_TYPE_SHOUTCAST && g_bPlayListFound) {
		CreateCustomGenreAndStationMenus();
	}
}

void CreateRadioBaseMenu()
{
	char szTranslation[64] = "";
	
	g_mRadioBaseMenu = new Menu(Handler_BaseRadioMenu);
	g_mRadioBaseMenu.SetTitle("SourceMod Radio:");
	
	if (g_iRadioPlayType == RADIO_PLAY_TYPE_SHOUTCAST && g_bShoutCastCustom) {
		Format(szTranslation, sizeof(szTranslation), "%T", "ShoutCast Stations", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("1", szTranslation);
		Format(szTranslation, sizeof(szTranslation), "%T", "Server Custom Stations", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("2", szTranslation);
		Format(szTranslation, sizeof(szTranslation), "%T", "Volume", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("3", szTranslation);
		Format(szTranslation, sizeof(szTranslation), "%T", "Turn Off", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("4", szTranslation);
	} else if (g_bUseGenres) {
		Format(szTranslation, sizeof(szTranslation), "%T", "Select Genre", LANG_SERVER);
		
		g_mRadioBaseMenu.AddItem("1", szTranslation);
		
		// Check if we need to add volume control to the base menu
		if (g_iRadioPlayType == RADIO_PLAY_TYPE_VOLUME_CONTROL || g_iRadioPlayType == RADIO_PLAY_TYPE_SHOUTCAST) {
			Format(szTranslation, sizeof(szTranslation), "%T", "Volume", LANG_SERVER);
			g_mRadioBaseMenu.AddItem("2", szTranslation);
		}
		
		Format(szTranslation, sizeof(szTranslation), "%T", "Turn Off", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("3", szTranslation);
	} else {
		Format(szTranslation, sizeof(szTranslation), "%T", "Select Station", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("1", szTranslation);
		if (g_cVarRadioPlayType.IntValue == RADIO_PLAY_TYPE_VOLUME_CONTROL) {
			Format(szTranslation, sizeof(szTranslation), "%T", "Volume", LANG_SERVER);
			g_mRadioBaseMenu.AddItem("2", szTranslation);
		}
		Format(szTranslation, sizeof(szTranslation), "%T", "Turn Off", LANG_SERVER);
		g_mRadioBaseMenu.AddItem("3", szTranslation);
	}
}

// Build the standard genres and stations menu
void CreateGenreAndStationMenus()
{
	char szTranslation[64];
	char szParentGenreID[12] = "";
	char szParentGenreName[128] = "";
	char szChildGenreID[12] = "";
	char szChildGenreName[128] = "";
	Genre primarygenre;
	Genre childgenre;
	int iIndex = 0;
	char szStationID[16] = "";
	char szStationName[64] = "";
	Station station;
	
	if (g_bUseGenres) {
		g_mRadioBaseGenresMenu = new Menu(Handler_SelectGenre);
		
		Format(szTranslation, sizeof(szTranslation), "%T:", "Select Genre", LANG_SERVER);
		g_mRadioBaseGenresMenu.SetTitle(szTranslation);
		
		// Cycle through all the genres and check which are parents
		for (iIndex = 0; iIndex < g_Genres.Count; iIndex++) {
			primarygenre = g_Genres.Items(iIndex);
			
			if (primarygenre.IsValid) {
				// Is this is a parent genre with children?
				if (primarygenre.m_bHasChildren) {
					// Add parent entry to base genres menu
					primarygenre.GetName(szParentGenreName, sizeof(szParentGenreName));
					IntToString(primarygenre.m_iID, szParentGenreID, sizeof(szParentGenreID));
					g_mRadioBaseGenresMenu.AddItem(szParentGenreID, szParentGenreName);
					
					// Initialize parent genre menu
					primarygenre.m_hMenu = new Menu(Handler_SelectGenre);
					SetMenuTitle(primarygenre.m_hMenu, szParentGenreName);
					
					// Append the child genres to the parent menus
					for (int iChildIndex = 0; iChildIndex < g_Genres.Count; iChildIndex++) {
						childgenre = g_Genres.Items(iChildIndex);
						
						if (childgenre.IsValid && (primarygenre.m_iID == childgenre.m_iParentID)) {
							IntToString(childgenre.m_iID, szChildGenreID, sizeof(szChildGenreID));
							childgenre.GetName(szChildGenreName, sizeof(szChildGenreName));
							AddMenuItem(primarygenre.m_hMenu, szChildGenreID, szChildGenreName);
						}
					}
					
					SetMenuExitBackButton(primarygenre.m_hMenu, true);
				} else if (primarygenre.m_iParentID == 0) {
					// This must be a base genre menu without children because it has no parent or children
					
					// Add an entry to the base menu but use the station handler on selection
					primarygenre.m_hMenu = new Menu(Handler_PlayRadioStation);
					primarygenre.GetName(szParentGenreName, sizeof(szParentGenreName));
					SetMenuTitle(primarygenre.m_hMenu, szParentGenreName);
					IntToString(primarygenre.m_iID, szParentGenreID, sizeof(szParentGenreID));
					g_mRadioBaseGenresMenu.AddItem(szParentGenreID, szParentGenreName);
				} else {
					// Everything else must be a genre menu with stations
					
					primarygenre.m_hMenu = new Menu(Handler_PlayRadioStation);
					Format(szTranslation, sizeof(szTranslation), "%T:", "Stations Menu Title", LANG_SERVER);
					SetMenuTitle(primarygenre.m_hMenu, szTranslation);
				}
			}
		}
		
		g_mRadioBaseGenresMenu.ExitBackButton = true;
		
		// Now add all the stations to the menus that match the genre ID
		for (iIndex = 0; iIndex < g_Stations.Count; iIndex++) {
			station = g_Stations.Items(iIndex);
			
			// Make sure the station index is valid and also the station ID
			if ((station.IsValid) && (station.m_iID > 0)) {
				primarygenre = g_Genres.Items(GetGenreIndexFromID(station.m_iGenreID));
				station.GetName(szStationName, sizeof(szStationName));
				
				if (primarygenre.IsValid && (station.m_iGenreID == 0 || !szStationName[0]))
					LogMessage("[SM-RADIO] Error: Station ID \"%d\" with name \"%s\" - Has an invalid genre ID or name", station.m_iID, szStationName);
				
				Format(szStationID, sizeof(szStationID), "%d", station.m_iID);
				AddMenuItem(primarygenre.m_hMenu, szStationID, szStationName);
			} else {
				LogMessage("[SM-RADIO] Error: Found an invalid station in CreateGenreAndStationMenus()");
			}
		}
		
		// Add the exit back button to all the station list menus aka ones without children
		for (iIndex = 0; iIndex < g_Genres.Count; iIndex++) {
			primarygenre = g_Genres.Items(iIndex);
			if (primarygenre.IsValid && !primarygenre.m_bHasChildren)
				SetMenuExitBackButton(primarygenre.m_hMenu, true);
		}
	} else {
		// Assume we're not using genres and append stations directly to the base menu
		g_mRadioStationsMenu = new Menu(Handler_PlayRadioStation);
		Format(szTranslation, sizeof(szTranslation), "%T:", "Stations Menu Title", LANG_SERVER);
		g_mRadioStationsMenu.SetTitle(szTranslation);
		
		for (iIndex = 0; iIndex < g_Stations.Count; iIndex++) {
			station = g_Stations.Items(iIndex);
			if (station.IsValid) {
				station.GetName(szStationName, sizeof(szStationName));
				Format(szStationID, sizeof(szStationID), "%d", station.m_iID);
				g_mRadioStationsMenu.AddItem(szStationID, szStationName);
			}
		}
		
		g_mRadioStationsMenu.ExitBackButton = true;
	}
}

// Same as above but using the custom genre/station method maps
void CreateCustomGenreAndStationMenus()
{
	char szTranslation[64];
	char szParentGenreID[12] = "";
	char szParentGenreName[128] = "";
	char szChildGenreID[12] = "";
	char szChildGenreName[128] = "";
	CustomGenre primarygenre;
	CustomGenre childgenre;
	int iIndex = 0;
	char szStationID[16] = "";
	char szStationName[64] = "";
	CustomStation station;
	
	if (g_bCustUseGenres)
	{
		g_mRadioCustomBaseGenresMenu = new Menu(Handler_CustomSelectGenre);
		
		Format(szTranslation, sizeof(szTranslation), "%T:", "Select Genre", LANG_SERVER);
		g_mRadioCustomBaseGenresMenu.SetTitle(szTranslation);
		
		// Cycle through all the genres and check which are parents
		for (iIndex = 0; iIndex < g_CustGenres.Count; iIndex++)
		{
			primarygenre = g_CustGenres.Items(iIndex);
			
			if (primarygenre.IsValid)
			{
				// Is this is a parent genre with children?
				if (primarygenre.m_bHasChildren)
				{
					// Add parent entry to base genres menu
					primarygenre.GetName(szParentGenreName, sizeof(szParentGenreName));
					IntToString(primarygenre.m_iID, szParentGenreID, sizeof(szParentGenreID));
					g_mRadioCustomBaseGenresMenu.AddItem(szParentGenreID, szParentGenreName);
					
					// Initialize parent genre menu
					primarygenre.m_hMenu = new Menu(Handler_CustomSelectGenre);
					SetMenuTitle(primarygenre.m_hMenu, szParentGenreName);
					
					// Append the child genres to the parent menus
					for (int iChildIndex = 0; iChildIndex < g_CustGenres.Count; iChildIndex++)
					{
						childgenre = g_CustGenres.Items(iChildIndex);
						
						if (childgenre.IsValid && (primarygenre.m_iID == childgenre.m_iParentID))
						{
							IntToString(childgenre.m_iID, szChildGenreID, sizeof(szChildGenreID));
							childgenre.GetName(szChildGenreName, sizeof(szChildGenreName));
							AddMenuItem(primarygenre.m_hMenu, szChildGenreID, szChildGenreName);
						}
					}
					
					SetMenuExitBackButton(primarygenre.m_hMenu, true);
				}
				// This is must be a base genre menu without children because it has no parent
				else if (primarygenre.m_iParentID == 0)
				{
					// Add an entry to the base menu but use the station handler on selection
					primarygenre.m_hMenu = new Menu(Handler_CustomPlayRadioStation);
					SetMenuTitle(primarygenre.m_hMenu, szParentGenreName);
					primarygenre.GetName(szParentGenreName, sizeof(szParentGenreName));
					IntToString(primarygenre.m_iID, szParentGenreID, sizeof(szParentGenreID));
					g_mRadioCustomBaseGenresMenu.AddItem(szParentGenreID, szParentGenreName);
				}
				// Everything else must be a genre menu with stations
				else
				{
					primarygenre.m_hMenu = new Menu(Handler_CustomPlayRadioStation);
					Format(szTranslation, sizeof(szTranslation), "%T:", "Stations Menu Title", LANG_SERVER);
					SetMenuTitle(primarygenre.m_hMenu, szTranslation);
				}
			}
		}
		
		g_mRadioCustomBaseGenresMenu.ExitBackButton = true;
		
		// Now add all the stations to the menus that match the genre ID
		for (iIndex = 0; iIndex < g_CustStations.Count; iIndex++)
		{
			station = g_CustStations.Items(iIndex);
			
			// Make sure the station index is valid and also the station ID
			if ((station.IsValid) && (station.m_iID > 0))
			{
				primarygenre = g_CustGenres.Items(GetCustomGenreIndexFromID(station.m_iGenreID));
				station.GetName(szStationName, sizeof(szStationName));
				
				if (primarygenre.IsValid && (station.m_iGenreID == 0 || !szStationName[0]))
					LogMessage("[SM-RADIO] Error: Custom Station ID \"%d\" with name \"%s\" - Has an invalid genre ID or name", station.m_iID, szStationName);
				
				Format(szStationID, sizeof(szStationID), "%d", station.m_iID);
				AddMenuItem(primarygenre.m_hMenu, szStationID, szStationName);
			}
			else
			{
				LogMessage("[SM-RADIO] Error: Found an invalid station in CreateCustomGenreAndStationMenus()");
			}
		}
		
		// Add the exit back button to all the station list menus aka ones without children
		for (iIndex = 0; iIndex < g_CustGenres.Count; iIndex++)
		{
			primarygenre = g_CustGenres.Items(iIndex);
			if (primarygenre.IsValid && !primarygenre.m_bHasChildren)
				SetMenuExitBackButton(primarygenre.m_hMenu, true);
		}
	}
	else
	{
		// Assume we're not using genres and append stations directly to the base menu
		g_mRadioStationsMenu = new Menu(Handler_CustomPlayRadioStation);
		Format(szTranslation, sizeof(szTranslation), "%T:", "Stations Menu Title", LANG_SERVER);
		g_mRadioStationsMenu.SetTitle(szTranslation);
		
		for (iIndex = 0; iIndex < g_CustStations.Count; iIndex++)
		{
			station = g_CustStations.Items(iIndex);
			if (station.IsValid)
			{
				station.GetName(szStationName, sizeof(szStationName));
				Format(szStationID, sizeof(szStationID), "%d", station.m_iID);
				g_mRadioStationsMenu.AddItem(szStationID, szStationName);
			}
		}
		
		g_mRadioStationsMenu.ExitBackButton = true;
	}
}

void CreateRadioVolumeMenu()
{
	char szTranslation[64] = "";
	
	g_mRadioVolumeMenu = new Menu(Handler_VolumeMenu);
	
	Format(szTranslation, sizeof(szTranslation), "%T:", "Volume Menu Title", LANG_SERVER);
	g_mRadioVolumeMenu.SetTitle(szTranslation);
	g_mRadioVolumeMenu.AddItem("0.01", "1%");
	g_mRadioVolumeMenu.AddItem("0.05", "5%");
	g_mRadioVolumeMenu.AddItem("0.10", "10%");
	g_mRadioVolumeMenu.AddItem("0.20", "20%");
	g_mRadioVolumeMenu.AddItem("0.40", "40%");
	g_mRadioVolumeMenu.AddItem("0.60", "60%");
	g_mRadioVolumeMenu.AddItem("0.80", "80%");
	g_mRadioVolumeMenu.AddItem("1.0", "100%");
	g_mRadioVolumeMenu.ExitBackButton = true;
}
