/*
 * SourceMod Radio
 *
 * Adds a radio player to SourceMod that supports direct station links or streaming audio
 *
 * Coded by dubbeh - www.dubbeh.net
 *
 * Licensed under the GPLv3
 *
 */

void GetCurretTrack(int iClient)
{
	char szClientStation[32] = "";
	int iStationID = 0;
	Station station;
	CustomStation custstation;
	Handle hHTTPRequest = INVALID_HANDLE;
	char szQueryURL[128] = "";
	char szCurrentTrack[128] = "";
	char szIsCustomShoutCast[8] = "";
	bool bIsCustomShoutCast = false;
	char szStreamURL[256] = "";
	
	GetClientCookie(iClient, g_hCookieSavedStation, szClientStation, sizeof(szClientStation));
	GetClientCookie(iClient, g_hCookieCustomShoutCast, szIsCustomShoutCast, sizeof(szIsCustomShoutCast));
	bIsCustomShoutCast = view_as<bool>(StringToInt(szIsCustomShoutCast));
	
	iStationID = StringToInt(szClientStation);
	
	if (iStationID > 0 && iStationID != -1)
	{
		if (g_iRadioPlayType == RADIO_PLAY_TYPE_SHOUTCAST && !bIsCustomShoutCast)
		{
			station = g_Stations.Items(GetStationIndexFromID(iStationID));
			
			if (station.IsValid)
			{
				if (station.m_iLastUpdateTime < (GetTime() + TRACK_UPDATE_TIME))
				{
					// Prepare the query URL and execute it
					Format(szQueryURL, sizeof(szQueryURL), "%s?sid=%s", g_szShoutCastQueryURL, szClientStation);
					if (STEAMWORKS_AVAILABLE())
					{
						if ((hHTTPRequest = SteamWorks_CreateHTTPRequest(k_EHTTPMethodGET, szQueryURL)) != INVALID_HANDLE)
						{
							if (!SteamWorks_SetHTTPRequestContextValue(hHTTPRequest, GetClientSerial(iClient)) || 
								!SteamWorks_SetHTTPCallbacks(hHTTPRequest, SteamWorks_OnTrackQueryComplete) || 
								!SteamWorks_SendHTTPRequest(hHTTPRequest))
							{
								CReplyToCommand(iClient, "%T", "SteamWorks Query Problem", LANG_SERVER);
								CloseHandle(hHTTPRequest);
							}
						}
					} else {
						CReplyToCommand(iClient, "%T", "Missing SteamWorks", LANG_SERVER);
					}
				} else {
					station.GetCurrentTrack(szCurrentTrack, sizeof(szCurrentTrack));
					CReplyToCommand(iClient, szCurrentTrack);
				}
			} else {
				LogMessage("[SM-RADIO] Error: Tried to load a station with an invalid index. Please check your stations list file.");
			}
		}
		// Special handling for custom ShoutCast stations
		else if (g_iRadioPlayType == RADIO_PLAY_TYPE_SHOUTCAST && bIsCustomShoutCast)
		{
			custstation = g_CustStations.Items(GetCustomStationIndexFromID(iStationID));
			if (custstation.IsValid)
			{
				if (custstation.m_iLastUpdateTime < (GetTime() + TRACK_UPDATE_TIME))
				{
					custstation.GetStreamURL(szStreamURL, sizeof(szStreamURL));
					
					// Prepare the query URL and execute it
					Format(szQueryURL, sizeof(szQueryURL), "%s?url=%s", g_szVolumeControlTrackQueryURL, szStreamURL);
					
					if (STEAMWORKS_AVAILABLE())
					{
						if ((hHTTPRequest = SteamWorks_CreateHTTPRequest(k_EHTTPMethodGET, szQueryURL)) != INVALID_HANDLE)
						{
							if (!SteamWorks_SetHTTPRequestContextValue(hHTTPRequest, GetClientSerial(iClient)) || 
								!SteamWorks_SetHTTPCallbacks(hHTTPRequest, SteamWorks_OnTrackQueryCompleteCustom) || 
								!SteamWorks_SendHTTPRequest(hHTTPRequest))
							{
								CReplyToCommand(iClient, "%T", "SteamWorks Query Problem", LANG_SERVER);
								CloseHandle(hHTTPRequest);
							}
						}
					} else {
						CReplyToCommand(iClient, "%T", "Missing SteamWorks", LANG_SERVER);
					}
				} else {
					custstation.GetCurrentTrack(szCurrentTrack, sizeof(szCurrentTrack));
					CReplyToCommand(iClient, szCurrentTrack);
				}
			} else {
				LogMessage("[SM-RADIO] Error: Tried to load a station with an invalid index. Please check your stations list file.");
			}
		}
		else if (g_iRadioPlayType == RADIO_PLAY_TYPE_VOLUME_CONTROL)
		{
			station = g_Stations.Items(GetStationIndexFromID(iStationID));
			if (station.IsValid)
			{
				if (station.m_iLastUpdateTime < (GetTime() + TRACK_UPDATE_TIME))
				{
					station.GetStreamURL(szStreamURL, sizeof(szStreamURL));
					
					// Prepare the query URL and execute it
					Format(szQueryURL, sizeof(szQueryURL), "%s?url=%s", g_szVolumeControlTrackQueryURL, szStreamURL);
					
					if (STEAMWORKS_AVAILABLE())
					{
						if ((hHTTPRequest = SteamWorks_CreateHTTPRequest(k_EHTTPMethodGET, szQueryURL)) != INVALID_HANDLE)
						{
							if (!SteamWorks_SetHTTPRequestContextValue(hHTTPRequest, GetClientSerial(iClient)) || 
								!SteamWorks_SetHTTPCallbacks(hHTTPRequest, SteamWorks_OnTrackQueryComplete) || 
								!SteamWorks_SendHTTPRequest(hHTTPRequest))
							{
								CReplyToCommand(iClient, "%T", "SteamWorks Query Problem", LANG_SERVER);
								CloseHandle(hHTTPRequest);
							}
						}
					} else {
						CReplyToCommand(iClient, "%T", "Missing SteamWorks", LANG_SERVER);
					}
				} else {
					station.GetCurrentTrack(szCurrentTrack, sizeof(szCurrentTrack));
					CReplyToCommand(iClient, szCurrentTrack);
				}
			} else {
				LogMessage("[SM-RADIO] Error: Tried to load a station with an invalid index. Please check your stations list file.");
			}
		} else {
			CReplyToCommand(iClient, "%T", "Track Query Unsupported", LANG_SERVER);
		}
	} else {
		CReplyToCommand(iClient, "Pick a station first to use the track query feature.");
	}
}

public SteamWorks_OnTrackQueryComplete(Handle hRequest, bool bFailure, bool bRequestSuccessful, EHTTPStatusCode eStatusCode, any iClientSerial)
{
	if (!bFailure && bRequestSuccessful && eStatusCode == k_EHTTPStatusCode200OK && iClientSerial > 0) {
		SteamWorks_GetHTTPResponseBodyCallback(hRequest, SteamWorks_TrackQueryWebResponse, iClientSerial);
	}
	
	CloseHandle(hRequest);
}

public SteamWorks_TrackQueryWebResponse(const char[] szResponseBuffer, any iClientSerial)
{
	int iClient = 0;
	char szStationID[12] = "";
	int iStationID = 0;
	Station station;
	
	iClient = GetClientFromSerial(iClientSerial);
	
	if (iClient && IsClientConnected(iClient) && IsClientInGame(iClient)) {
		GetClientCookie(iClient, g_hCookieSavedStation, szStationID, sizeof(szStationID));
		iStationID = StringToInt(szStationID);
		
		station = g_Stations.Items(GetStationIndexFromID(iStationID));
		station.SetCurrentTrack(szResponseBuffer);
		station.m_iLastUpdateTime = GetTime();
		CPrintToChat(iClient, szResponseBuffer);
	} else {
		LogMessage("[SM-RADIO] SteamWorks_TrackQueryWebResponse() GetClientSerial returned 0. Maybe the client disconnected.");
	}
}

public SteamWorks_OnTrackQueryCompleteCustom(Handle hRequest, bool bFailure, bool bRequestSuccessful, EHTTPStatusCode eStatusCode, any iClientSerial)
{
	if (!bFailure && bRequestSuccessful && eStatusCode == k_EHTTPStatusCode200OK && iClientSerial > 0) {
		SteamWorks_GetHTTPResponseBodyCallback(hRequest, SteamWorks_TrackQueryWebResponseCustom, iClientSerial);
	}
	
	CloseHandle(hRequest);
}

public SteamWorks_TrackQueryWebResponseCustom(const char[] szResponseBuffer, any iClientSerial)
{
	int iClient = 0;
	char szStationID[12] = "";
	int iStationID = 0;
	CustomStation custstation;
	
	iClient = GetClientFromSerial(iClientSerial);
	
	if (iClient && IsClientConnected(iClient) && IsClientInGame(iClient)) {
		GetClientCookie(iClient, g_hCookieSavedStation, szStationID, sizeof(szStationID));
		iStationID = StringToInt(szStationID);
		
		custstation = g_CustStations.Items(GetCustomStationIndexFromID(iStationID));
		custstation.SetCurrentTrack(szResponseBuffer);
		custstation.m_iLastUpdateTime = GetTime();
		CPrintToChat(iClient, szResponseBuffer);
	} else {
		LogMessage("[SM-RADIO] SteamWorks_TrackQueryWebResponse() GetClientSerial returned 0. Maybe the client disconnected.");
	}
}
